c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine is_in_box(dim,x0,box_dimensions,in_box)
      implicit none
      include 'max.inc'
c
c     Purpose: to check whether or not a given position is in the box
c
c     Input:
c       + dim: space dimension
c       + x0: position to check
c       + box_dimensions(i,j):
c         - i: index of the dimension (1:x, 2:y, 3:z)
c         - j: index of limit (1: lower, 2: higher)
c
c     Output:
c       + in_box: true if x0 is in the box, false otherwise
c
c     I/O
      integer dim
      double precision x0(1:Ndim_mx)
      double precision box_dimensions(1:Ndim_mx,1:2)
      logical in_box
c     temp
      integer i
c     label
      character*(Nchar_mx) label
      label='subroutine is_in_box'

      in_box=.true.
      do i=1,dim
         if ((x0(i).lt.box_dimensions(i,1)).or.
     &        (x0(i).gt.box_dimensions(i,2))) then
            in_box=.false.
c     Debug
c            write(*,*) 'x0(',i,')=',x0(i)
c            if (x0(i).lt.box_dimensions(i,1)) then
c               write(*,*) '< box_dimensions(',i,',1)=',
c     &              box_dimensions(i,1)
c            endif
c            if (x0(i).gt.box_dimensions(i,2)) then
c               write(*,*) '> box_dimensions(',i,',2)=',
c     &              box_dimensions(i,2)
c            endif
c     Debug
            goto 111
         endif
      enddo ! i
 111  continue
      
      return
      end



      subroutine is_on_surface(dim,x0,box_dimensions,
     &     on_surface,surface_index1,surface_index2)
      implicit none
      include 'max.inc'
c
c     Purpose: to check whether or not a given position is on the surface of the box
c
c     Input:
c       + dim: space dimension
c       + x0: position to check
c       + box_dimensions(i,j):
c         - i: index of the dimension (1:x, 2:y, 3:z)
c         - j: index of limit (1: lower, 2: higher)
c
c     Output:
c       + on_surface: true if x0 is on the surface the box, false otherwise
c       + (surface_index1,surface_index2): indexes of the surface "x0" is located on
c
c     I/O
      integer dim
      double precision x0(1:Ndim_mx)
      double precision box_dimensions(1:Ndim_mx,1:2)
      logical on_surface
      integer surface_index1
      integer surface_index2
c     temp
      integer i,j
      logical x0_is_in_box
c     label
      character*(Nchar_mx) label
      label='subroutine is_on_surface'

      on_surface=.false.
      call is_in_box(dim,x0,box_dimensions,x0_is_in_box)
      if (x0_is_in_box) then
         do i=1,dim
            do j=1,2
               if (dabs(box_dimensions(i,j)-x0(i)).lt.1.0D-10) then
                  on_surface=.true.
                  surface_index1=i
                  surface_index2=j
                  goto 111
               endif
            enddo               ! j
         enddo                  ! i
      endif                     ! x0_is_in_box
 111  continue

      return
      end



      subroutine test_r(r,label)
      implicit none
      include 'max.inc'
c
c     Purpose: to check the value of "r" in positive
c
c     Input:
c       + r: value to check
c       + label: name of the calling routine
c
c     I/O
      double precision r
      character*(Nchar_mx) label
c     temp

      if (r.lt.0.0D+0) then
         call error(label)
         write(*,*) 'r=',r
         stop
      endif

      return
      end



      subroutine test_theta(theta,label)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to check the value of "theta" is in [-pi/2,pi/2]
c
c     Input:
c       + theta: value of theta
c       + label: name of the calling routine
c
c     I/O
      double precision theta
      character*(Nchar_mx) label
c     temp

      if ((theta.gt.pi/2.0D+0).or.(theta.lt.-pi/2.0D+0)) then
         call error(label)
         write(*,*) 'theta=',theta
         stop
      endif

      return
      end



      subroutine test_phi(phi,label)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to check the value of "phi" is in [0,2*pi]
c
c     Input:
c       + phi: value of phi
c       + label: name of the calling routine
c
c     I/O
      double precision phi
      character*(Nchar_mx) label
c     temp
      double precision epsilon
      parameter(epsilon=1.0D-6)

      if ((phi.lt.-epsilon).or.
     &     (phi-2.0D+0*pi.gt.epsilon)) then
         call error(label)
         write(*,*) 'phi=',phi
         stop
      endif

      return
      end
