c     Copyright (C)
c     2021 Mésotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      program sw_flux
      implicit none
      include 'max.inc'
      include 'param.inc'
c     
c     The purpose of this program is to validate the null-collision algorithm that was
c     established for radiative transfer computations in the presence of a semi-transparent
c     medium, the source being a laser sheet, thin compared to all dimensions of the system.
c     This algorithm was implemented into the htrdr-combustion application.
c     
c     Reference:
c     "Null-collision meshless Monte Carlo - A new reverse Monte Carlo algorithm designed for laser-source emission in absorbing/scattering inhomogeneous media"
c     M.Sans, M.El Hafi, V.Eymet, V.Forest, R.Fournier, N.Villefranque
c     Journal of Quantitative Spectroscopy and Radiative Transfer, Volume 271, September 2021
c     
c     DOI:
c     https://www.sciencedirect.com/science/article/abs/pii/S0022407321002181
c
c     Variables
      integer dim
      character*(Nchar_mx) datafile
      double precision box_volume(1:Ndim_mx,1:2)
      double precision laser_volume(1:Ndim_mx,1:2)
      double precision emission_surface(1:Ndim_mx,1:2)
      double precision integration_surface(1:Ndim_mx,1:2)
      double precision d
      double precision phi0
      complex m
      double precision kf
      double precision Dp
      double precision Np
      double precision fv_max
      double precision norm_factor
      integer pf
      double precision g_HG
      logical direct_computation
      logical reverse_computation
      integer Nevent
      integer seed
c     Debug
      integer nc_tr_version
      double precision alpha,phase_function
c     Debug
c     label
      character*(Nchar_mx) label
      label='program sw_flux'

c     dimension of space
      dim=3
c     reading input data
      datafile='./data.in'

      call read_data(dim,datafile,box_volume,laser_volume,
     &     emission_surface,integration_surface,d,
     &     phi0,m,kf,Dp,Np,
     &     pf,g_HG,direct_computation,reverse_computation,Nevent)
      nc_tr_version=1

      
c     random seed
      call read_seed(seed)
      call zufalli(seed)

c     Initialization of RDG-FA parameters
      call rdgfa_init(m,kf,Dp,Np,norm_factor)
      
c     MC integration
c     
      if (direct_computation) then
         call sw_direct(dim,box_volume,laser_volume,
     &        emission_surface,integration_surface,
     &        d,phi0,nc_tr_version,pf,g_HG,norm_factor,Nevent)
      endif                     ! direct_computation
c     
      if (reverse_computation) then
         call sw_reverse(dim,box_volume,laser_volume,
     &        emission_surface,integration_surface,
     &        d,phi0,nc_tr_version,pf,g_HG,norm_factor,Nevent)
      endif                     ! reverse_computation
      
      end
